//
//  ViewController.swift
//  Concentration
//
//  Created by Sameh on 8/28/19.
//  Copyright © 2019 Sameh. All rights reserved.
//

import UIKit

class ViewController: UIViewController {

    private lazy var game = Concentration(numberOfPairsOfCards: numberOfPairsOfCards)
    
    private var numberOfPairsOfCards : Int {
        return (cardButtons.count + 1) / 2
    }
    
    private(set) var flipCount = 0 {
        didSet {
            updateFlipCountLabel()
        }
    }

    private func updateFlipCountLabel () {
        let attributes: [NSAttributedString.Key: Any] = [
                        .strokeColor : #colorLiteral(red: 1, green: 0.5781051517, blue: 0, alpha: 1),
                        .strokeWidth : 5.0
                        ]
    
        let attributedString = NSAttributedString(string: "Flips: \(flipCount)",
                                                attributes: attributes)
    
        flipCountLabel.attributedText = attributedString
    }
    
    @IBOutlet private var cardButtons: [UIButton]!
    
    @IBOutlet private weak var flipCountLabel: UILabel! {
        didSet {
            updateFlipCountLabel()
        }
    }

    private var emojiChoices = "👻🎃☠️😈🍎🧟‍♂️🍬🍫🍭"
    
    private var emoji = [Card : String]()

    private func emoji(for card: Card) -> String {
//        if let chosenEmoji = emoji[card.identifier] {
//            return emoji[card.identifier]!
//        } else {
//            return "?"
//        }
        if emoji[card] == nil , emojiChoices.count > 0 {
                let randomStringIndex = emojiChoices.index(emojiChoices.startIndex, offsetBy: emojiChoices.count.arc4random)
                let character = emojiChoices.remove(at: randomStringIndex)
                emoji[card] = String(character)
        }
        return emoji[card] ?? "?"
    }
    
    @IBAction private func touchCard(_ sender: UIButton) {
        flipCount += 1
        if let cardNumber = cardButtons.firstIndex(of: sender) {
            game.chooseCard(at: cardNumber)
            updateViewFromModel()
        } else {
            print("Chosen card not found in cardButtons")
        }
    }
    
    private func updateViewFromModel() {
        for index in cardButtons.indices {
            let button = cardButtons[index]
            let card = game.cards[index]
            if card.isFaceUp {
                button.setTitle(emoji(for: card), for: UIControl.State.normal)
                button.backgroundColor = #colorLiteral(red: 1, green: 1, blue: 1, alpha: 1)
            } else {
                button.setTitle("", for: UIControl.State.normal)
                if card.isMatched {
                    button.backgroundColor = #colorLiteral(red: 0, green: 0, blue: 0, alpha: 0)
                } else {
                    button.backgroundColor = #colorLiteral(red: 1, green: 0.5781051517, blue: 0, alpha: 1)
                }
            }
        }
    }
}


extension Int {
    var arc4random: Int {
        if (self > 0) {
            return Int(arc4random_uniform(UInt32(self)))
        } else if (self < 0) {
            return -Int(arc4random_uniform(UInt32(-self)))
        } else {
            return 0
        }
    }
}
